use arduino_hal::{
    delay_ms,
    hal::port::Dynamic,
    port::{
        mode::{Input, Output},
        Pin,
    },
};

type Key = [Option<(usize, usize)>; 6];

#[macro_export]
macro_rules! pins_array {
    ("cols", $per:expr, $($pins:tt), * ) => {
        [ $($per.$pins.into_pull_up_input().downgrade().forget_imode()), * ]
    };
    ("rows", $per:expr, $($pins:tt), * ) => {
        [ $($per.$pins.into_output_high().downgrade()), * ]
    }
}

pub struct KeyboardMatrix<const C: usize, const R: usize> {
    cols: [Pin<Input>; C],
    rows: [Pin<Output, Dynamic>; R],
}

impl<const C: usize, const R: usize> KeyboardMatrix<C, R> {
    #[inline]
    pub fn new(cols: [Pin<Input, Dynamic>; C], rows: [Pin<Output, Dynamic>; R]) -> Self {
        Self { cols, rows }
    }

    pub fn scan(&mut self) -> Key {
        let mut key_pos = [None; 6];
        let mut index = 0usize;
        for (i, row) in self.rows.iter_mut().enumerate() {
            row.set_low();
            delay_ms(1);
            for (j, col) in self.cols.iter().enumerate() {
                if col.is_low() && index < 6 {
                    key_pos[index] = Some((i, j));
                    index += 1;
                }
            }
            row.set_high();
        }
        key_pos
    }
}
